@@ -1,25 +1,24 @@
-/***************************************************************************
- *   Copyright (C) 2004-5 by Enrico Ros <eros.kde@email.it>                *
- *   Copyright (C) 2005   by Piotr Szymanski <niedakh@gmail.com>           *
- *   Copyright (C) 2008   by Albert Astals Cid <aacid@kde.org>             *
- *   Copyright (C) 2017   Klarälvdalens Datakonsult AB, a KDAB Group       *
- *                        company, info@kdab.com. Work sponsored by the    *
- *                        LiMux project of the city of Munich              *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- ***************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2004-5 Enrico Ros <eros.kde@email.it>
+    SPDX-FileCopyrightText: 2005 Piotr Szymanski <niedakh@gmail.com>
+    SPDX-FileCopyrightText: 2008 Albert Astals Cid <aacid@kde.org>
+
+    Work sponsored by the LiMux project of the city of Munich:
+    SPDX-FileCopyrightText: 2017 Klarälvdalens Datakonsult AB a KDAB Group company <info@kdab.com>
+
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 #ifndef _OKULAR_GENERATOR_H_
 #define _OKULAR_GENERATOR_H_
 
+#include "action.h"
 #include "document.h"
 #include "fontinfo.h"
 #include "global.h"
 #include "okularcore_export.h"
 #include "pagesize.h"
+#include "signatureutils.h"
 
 #include <QList>
 #include <QObject>
@@ -27,7 +26,6 @@
 #include <QSizeF>
 #include <QString>
 #include <QVariant>
-#include <QVector>
 
 #include <KPluginFactory>
 #include <QMimeType>
@@ -232,7 +230,7 @@ public:
      *
      * @returns true on success, false otherwise.
      */
-    virtual bool loadDocument(const QString &fileName, QVector<Page *> &pagesVector);
+    virtual bool loadDocument(const QString &fileName, QList<Page *> &pagesVector);
 
     /**
      * Loads the document from the raw data @p fileData and fills the
@@ -244,7 +242,7 @@ public:
      *
      * @returns true on success, false otherwise.
      */
-    virtual bool loadDocumentFromData(const QByteArray &fileData, QVector<Page *> &pagesVector);
+    virtual bool loadDocumentFromData(const QByteArray &fileData, QList<Page *> &pagesVector);
 
     /**
      * Loads the document with the given @p fileName and @p password and fills the
@@ -256,7 +254,7 @@ public:
      *
      * @returns a LoadResult defining the result of the operation
      */
-    virtual Document::OpenResult loadDocumentWithPassword(const QString &fileName, QVector<Page *> &pagesVector, const QString &password);
+    virtual Document::OpenResult loadDocumentWithPassword(const QString &fileName, QList<Page *> &pagesVector, const QString &password);
 
     /**
      * Loads the document from the raw data @p fileData and @p password and fills the
@@ -270,7 +268,7 @@ public:
      *
      * @returns a LoadResult defining the result of the operation
      */
-    virtual Document::OpenResult loadDocumentFromDataWithPassword(const QByteArray &fileData, QVector<Page *> &pagesVector, const QString &password);
+    virtual Document::OpenResult loadDocumentFromDataWithPassword(const QByteArray &fileData, QList<Page *> &pagesVector, const QString &password);
 
     /**
      * Describes the result of an swap file operation.
@@ -291,7 +289,7 @@ public:
      *
      * @since 1.3
      */
-    virtual SwapBackingFileResult swapBackingFile(const QString &newFileName, QVector<Okular::Page *> &newPagesVector);
+    virtual SwapBackingFileResult swapBackingFile(const QString &newFileName, QList<Okular::Page *> &newPagesVector);
 
     /**
      * This method is called when the document is closed and not used
@@ -307,6 +305,30 @@ public:
      */
     virtual bool canGeneratePixmap() const;
 
+    virtual bool canSign() const;
+
+    /**
+     * Signs the document
+     * \since 25.08
+     * @return pair of signing result and a detailed/advanced error message/code in case of errors
+     */
+    virtual std::pair<SigningResult, QString> sign(const NewSignatureData &data, const QString &rFilename);
+
+    virtual CertificateStore *certificateStore() const;
+
+    /**
+     * @brief sets the active certificate backend
+     * @param backend
+     * @since 25.08
+     */
+    void setActiveCertificateBackend(Okular::CertificateInfo::Backend backend);
+    /**
+     * @brief returns the active certificate backend, defaults to unknown
+     * @param backend
+     * @since 25.08
+     */
+    Okular::CertificateInfo::Backend activeCertificateBackend() const;
+
     /**
      * This method can be called to trigger the generation of
      * a new pixmap as described by @p request.
@@ -357,6 +379,32 @@ public:
      */
     virtual const QList<EmbeddedFile *> *embeddedFiles() const;
 
+    /**
+     * This enum identifies default page layouts.
+     *
+     * @since 24.12
+     */
+    enum PageLayout {
+        NoLayout = -1, ///< Layout not specified
+        SinglePage,    ///< Display a single page
+        TwoPageLeft,   ///< Display the pages in two columns, with odd-numbered pages on the left
+        TwoPageRight   ///< Display the pages in two columns, with odd-numbered pages on the right
+    };
+
+    /**
+     * This method returns the default page layout.
+     *
+     * @since 24.12
+     */
+    virtual PageLayout defaultPageLayout() const;
+
+    /**
+     * This method returns if the default page layout is continuous.
+     *
+     * @since 24.12
+     */
+    virtual bool defaultPageContinuous() const;
+
     /**
      * This enum identifies the metric of the page size.
      */
@@ -395,26 +443,7 @@ public:
     /**
      * This method is called to print the document to the given @p printer.
      */
-    virtual bool print(QPrinter &printer);
-
-    /**
-     * Possible print errors
-     * @since 0.11 (KDE 4.5)
-     */
-    enum PrintError {
-        NoPrintError, ///< There was no print error
-        UnknownPrintError,
-        TemporaryFileOpenPrintError,
-        FileConversionPrintError,
-        PrintingProcessCrashPrintError,
-        PrintingProcessStartPrintError,
-        PrintToFilePrintError,
-        InvalidPrinterStatePrintError,
-        UnableToFindFilePrintError,
-        NoFileToPrintError,
-        NoBinaryToPrintError,
-        InvalidPageSizePrintError ///< @since 0.18.2 (KDE 4.12.2)
-    };
+    virtual Document::PrintError print(QPrinter &printer);
 
     /**
      * This method returns the meta data of the given @p key with the given @p option
@@ -448,9 +477,10 @@ public:
     /**
      * Update DPI of the generator
      *
-     * @since 0.19 (KDE 4.13)
+     * @since 0.19 (old signature)
+     * @since 22.04 (new signature)
      */
-    void setDPI(const QSizeF &dpi); // TODO remove the & when we do a BIC change elsewhere
+    void setDPI(const QSizeF dpi);
 
     /**
      * Returns the 'layers model' object of the document or NULL if
@@ -461,9 +491,23 @@ public:
     virtual QAbstractItemModel *layersModel() const;
 
     /**
-     * Calls the backend to execute an BackendOpaqueAction
+     * Calls the backend to execute a BackendOpaqueAction @p action and returns BackendOpaqueAction result
      */
-    virtual void opaqueAction(const BackendOpaqueAction *action);
+    virtual BackendOpaqueAction::OpaqueActionResult opaqueAction(const BackendOpaqueAction *action);
+
+    /**
+     * Frees the contents of the opaque action (if any);
+     *
+     * @since 22.04
+     */
+    virtual void freeOpaqueActionContents(const BackendOpaqueAction &action);
+
+    /**
+     * Retrieves the additional document action for the specified @p type .
+     *
+     * @since 24.08
+     */
+    virtual Okular::Action *additionalDocumentAction(Document::DocumentAdditionalActionType type);
 
 Q_SIGNALS:
     /**
@@ -561,12 +605,6 @@ protected:
      */
     QVariant documentMetaData(const DocumentMetaDataKey key, const QVariant &option = QVariant()) const;
 
-    /**
-     * Request a meta data of the Document, if available, like an internal
-     * setting.
-     */
-    OKULARCORE_DEPRECATED QVariant documentMetaData(const QString &key, const QVariant &option = QVariant()) const;
-
     /**
      * Return the pointer to a mutex the generator can use freely.
      */
@@ -587,15 +625,15 @@ protected:
      */
     QSizeF dpi() const;
 
-protected Q_SLOTS:
     /**
      * Gets the font data for the given font
      *
-     * @since 0.8 (KDE 4.1)
+     * @since 0.8 (old signature)
+     * @since 22.04 (new signature)
      */
-    // TODO Make it return a QByteArray and be virtual when a BIC change happens somewhere else
-    void requestFontData(const Okular::FontInfo &font, QByteArray *data);
+    virtual QByteArray requestFontData(const Okular::FontInfo &font);
 
+protected Q_SLOTS:
     /**
      * This method can be called to trigger a partial pixmap update for the given request
      * Make sure you call it in a way it's executed in the main thread.
@@ -604,13 +642,6 @@ protected Q_SLOTS:
     void signalPartialPixmapRequest(Okular::PixmapRequest *request, const QImage &image);
 
 protected:
-    /**
-     * Returns the last print error in case print() failed
-     * @since 0.11 (KDE 4.5)
-     */
-    // TODO Make print() return a PrintError instead of bool and remove this function when a BIC change happens somewhere else
-    Q_INVOKABLE Okular::Generator::PrintError printError() const;
-
     /// @cond PRIVATE
     Generator(GeneratorPrivate &dd, QObject *parent, const QVariantList &args);
     Q_DECLARE_PRIVATE(Generator)
@@ -641,12 +672,13 @@ public:
      *
      * @param observer The observer.
      * @param pageNumber The page number.
-     * @param width The width of the page.
-     * @param height The height of the page.
+     * @param width The width of the page in logical pixels.
+     * @param height The height of the page in logical pixels.
+     * @param dpr Device pixel ratio of the screen that the pixmap is intended for.
      * @param priority The priority of the request.
      * @param features The features of generation.
      */
-    PixmapRequest(DocumentObserver *observer, int pageNumber, int width, int height, int priority, PixmapRequestFeatures features);
+    PixmapRequest(DocumentObserver *observer, int pageNumber, int width, int height, qreal dpr, int priority, PixmapRequestFeatures features);
 
     /**
      * Destroys the pixmap request.
@@ -796,7 +828,6 @@ private:
 
 }
 
-Q_DECLARE_METATYPE(Okular::Generator::PrintError)
 Q_DECLARE_METATYPE(Okular::PixmapRequest *)
 
 #define OkularGeneratorInterface_iid "org.kde.okular.Generator"
